function MedLimitedDoses(genName, trName, rt, freq, units, wtDose, doses) {
    Med.apply(this, [ genName, trName, rt, freq, units, wtDose, 1 ]);
    this.doses = doses ? doses : [];
}

function MedDoseRange(genName, trName, rt, freq, units, wtDose, roundTo, dose, doseMax) {
    Med.apply(this, [ genName, trName, rt, freq, units, wtDose, roundTo, dose ]);
    this.doseMax = doseMax ? doseMax : 0;
}

function MedLiquid(genName, trName, rt, freq, units, wtDose, roundTo, maxDose, tspStr) {
    Med.apply(this, [ genName, trName, rt, freq, units, wtDose, roundTo, maxDose ]);
    this.stdTspStrength = tspStr ? tspStr : 0;
}

function MedLiquidMultipleStrengths(genName, trName, rt, freq, units, wtDose, roundTo, maxDose, tspStrArr) {
    var defaultTspStr = tspStrArr && tspStrArr.length > 0 ? tspStrArr[0] : 0;
    MedLiquid.apply(this, [ genName, trName, rt, freq, units, wtDose, roundTo, maxDose, defaultTspStr ]);
    this.tspStrArr = tspStrArr ? tspStrArr : [];
    this.n = 0;
}

function MedLiquidDouble() {}

function MedMultipleStrengths(genName, trName, rt, freq, units, wtDose, roundTo, maxDose, strArr) {
    var defaultStr = strArr && strArr.length > 0 ? strArr[0] : 0;
    MedLiquid.apply(this, [ genName, trName, rt, freq, units, wtDose, roundTo, maxDose, defaultStr ]);
    this.setStrArr(strArr);
}

function MedBland(genName, trName, dosing) {
    Med.apply(this, [ genName, trName ]);
    this.dosing = dosing ? dosing : "";
    this.curDosing = "";
    this.setCurDosing();
}

var Med = function(a, b, c, d, e, f, g, h) {
    this.genericName = a ? a : "";
    this.tradeName = b ? b : "";
    this.route = c ? c : "";
    this.freq = d ? d : "";
    this.units = e ? e : "";
    this.stdWtDose = f ? f : 0;
    this.roundTo = g ? g : false;
    this.stdDose = h ? h : 99999;
    this.indication = "";
    this.notes = "";
    this.notes2 = "";
    this.bPedsDosing = true;
    this.tags = "";
    this.pregnancy = "";
    this.lactation = "";
    this.source = "";
    this.mech = "";
    this.halflife = "";
    this.special = "";
};

Med.prototype.getID = function() {
    return (this.genericName + this.indication).replace(/\W+/g, "");
};

Med.prototype.getIndication = function() {
    return 0 == this.indication.length && 0 == this.notes.length ? "" : '<p class="sm-fade">' + this.indication + "<br />" + this.notes + "</p>";
};

Med.prototype.listify = function(wt) {
    return "<li>" + this.printRx(wt) + this.getIndication() + "</li>";
};

Med.prototype._formatDose = function(dose) {
    var arr = this.units.split("/");
    var units = arr[0];
    var time = arr.length > 1 ? arr[1] : "";
    if (dose >= 1e3 && "mcg" == units) {
        dose /= 1e3;
        units = "mg";
    }
    if (dose >= 1e3 && ("mg" == units || "mL" == units)) {
        dose /= 1e3;
        "mg" == units ? units = "g" : "mL" == units && (units = "L");
    }
    if (.001 >= dose && "mg" == units) {
        dose = 1e3 * dose;
        units = "mcg";
    } else if (.01 >= dose && "g" == units || "L" == units) {
        dose = 1e3 * dose;
        "g" == units ? units = "mg" : "L" == units && (units = "mL");
    }
    var str = dose + " " + units;
    time.length > 0 && (str += "/" + time);
    return str;
};

Med.prototype._roundToNearest = function(dose) {
    if (!this.roundTo || this.roundTo <= 0) return dose;
    var amount = Math.round(dose * this.roundTo) / this.roundTo;
    var mod = dose % this.roundTo;
    amount = mod < this.roundTo / 2 ? dose - mod : dose + this.roundTo - mod;
    amount = Math.round(10 * amount) / 10;
    amount < this.roundTo && (amount = this.roundTo);
    return amount;
};

Med.prototype._calcDose = function(weight) {
    var dose = weight * this.stdWtDose;
    !isNaN(this.roundTo) && this.roundTo > 0 && (dose = this._roundToNearest(dose));
    return dose;
};

Med.prototype._updateCurDosing = function() {};

Med.prototype.calcDose = function(weight) {
    if (isNaN(weight) || 0 > weight) return "";
    this._updateCurDosing();
    var isWtBased = true;
    var dose = this._calcDose(weight);
    if (this.stdDose > 0 && dose > this.stdDose) {
        dose = this.stdDose;
        isWtBased = false;
    }
    var str = this._formatDose(dose);
    isWtBased && (str += " (" + this.stdWtDose + this.units + "/kg)");
    return str;
};

Med.prototype.printRx = function(weight) {
    var rxStr = "<span id='" + this.getID() + "-name' class='generic'>" + this.genericName + "</span> ";
    this.tradeName.length > 0 && (rxStr += "(" + this.tradeName + "<sup>&reg;</sup>) ");
    if (50 > weight && !this.bPedsDosing) return '<span class="fade">' + rxStr + "- No pediatric dosing</span>";
    rxStr += this.calcDose(weight);
    rxStr += " " + this.route;
    this.freq.length > 0 && (rxStr += " " + this.freq);
    return rxStr;
};

MedLimitedDoses.prototype = new Med();

MedLimitedDoses.prototype.constructor = MedLimitedDoses;

MedLimitedDoses.prototype._roundToNearest = function(dose) {
    for (var i = this.doses.length - 1; i >= 0; --i) {
        if (0 == i) return this.doses[0];
        if (dose > this.doses[i]) return this.doses[i];
    }
};

MedLimitedDoses.prototype.calcDose = function(weight) {
    if (isNaN(weight) || 0 > weight) return "";
    var dose = this._calcDose(weight);
    return dose + this.units;
};

MedDoseRange.prototype = new Med();

MedDoseRange.prototype.constructor = MedDoseRange;

MedDoseRange.prototype.printRx = function() {
    return "Med with a range of appropriate doses for various weight groups";
};

MedLiquid.prototype = new Med();

MedLiquid.prototype.constructor = MedLiquid;

MedLiquid.prototype.printTsp = function(dose) {
    if (this.stdTspStrength <= 0) return "";
    var amt = Math.round(dose / this.stdTspStrength * 10) / 10;
    return amt + " tsp (" + this.stdTspStrength + " " + this.units + "/5mL)";
};

MedLiquid.prototype.calcDose = function(weight) {
    if (isNaN(weight) || 0 > weight) return "";
    var isWtBased = true;
    var dose = this._calcDose(weight);
    if (this.stdDose > 0 && dose > this.stdDose) {
        dose = this.stdDose;
        isWtBased = false;
    }
    var str = this.printTsp(dose);
    isWtBased && (str += " [" + this.stdWtDose + this.units + "/kg]");
    return str;
};

MedLiquidMultipleStrengths.prototype = new MedLiquid();

MedLiquidMultipleStrengths.prototype.constructor = MedLiquidMultipleStrengths;

MedLiquidMultipleStrengths.prototype.printTsp = function(dose) {
    var newStrength;
    try {
        newStrength = $("#" + this.getID() + "-sel").val();
    } catch (e) {}
    newStrength && (this.stdTspStrength = newStrength);
    if (this.stdTspStrength <= 0) return "";
    var amt = Math.round(dose / this.stdTspStrength * 10) / 10;
    var strengths = '<select id="' + this.getID() + '-sel"';
    strengths += ' data-native-menu="false">';
    for (var i = 0; i < this.tspStrArr.length; ++i) {
        strengths += '<option value="' + this.tspStrArr[i] + '"';
        this.tspStrArr[i] == this.stdTspStrength && (strengths += ' selected="selected"');
        strengths += ">" + this.tspStrArr[i] + this.units + "/5mL</option>";
    }
    strengths += "</select>";
    return amt + " tsp " + strengths;
};

MedLiquidDouble.prototype = new MedLiquid();

MedLiquidDouble.prototype.constructor = MedLiquidDouble;

MedLiquidDouble.prototype.printRx = function() {
    return "Med with liquid-only formulation of two ingredients";
};

MedMultipleStrengths.prototype = new Med();

MedMultipleStrengths.prototype.constructor = MedMultipleStrengths;

MedMultipleStrengths.prototype.setStrArr = function(newStrArr) {
    this.strArr = newStrArr;
    this.n = 0;
    if (this.strArr && this.strArr.length > 0) try {
        this.stdWtDose = this.strArr[0][1];
        this.stdDose = this.strArr[0][2];
        this.strArr[0].length >= 4 && this.strArr[0][3].length > 0 && (this.units = this.strArr[0][3]);
        this.strArr[0].length >= 5 && this.strArr[0][4].length > 0 && (this.route = this.strArr[0][4]);
        this.strArr[0].length >= 6 && this.strArr[0][5].length > 0 && (this.freq = this.strArr[0][5]);
    } catch (e) {}
};

MedMultipleStrengths.prototype._updateCurDosing = function() {
    var val;
    try {
        val = $("#" + this.getID() + "-sel").val().split("|");
        this.n = val[0];
        val[1].length > 0 && (this.stdWtDose = parseFloat(val[1]));
        val.length >= 3 && val[2].length > 0 && (this.stdDose = parseFloat(val[2]));
        val.length >= 4 && val[3].length > 0 && (this.units = val[3]);
        val.length >= 5 && val[4].length > 0 && (this.route = val[4]);
        val.length >= 6 && val[5].length > 0 && (this.freq = val[5]);
        if (this.stdWtDose < 0 || this.stdDose < 0) throw "Cannot be negative!";
    } catch (e) {}
};

MedMultipleStrengths.prototype._formatDose = function(dose) {
    if (0 >= dose) return "";
    var amt = Med.prototype._formatDose.call(this, dose);
    var strengths = '<select id="' + this.getID() + '-sel"';
    strengths += ' data-native-menu="false">';
    for (var i = 0; i < this.strArr.length; ++i) {
        strengths += '<option value="' + i + "|";
        for (var j = 1; j < this.strArr[i].length; ++j) strengths += this.strArr[i][j] + "|";
        strengths += '"';
        i == this.n && (strengths += ' selected="selected"');
        strengths += ">" + this.strArr[i][0] + "</option>";
    }
    strengths += "</select>";
    return strengths + " " + amt;
};

MedBland.prototype = new Med();

MedBland.prototype.constructor = MedBland;

MedBland.prototype.setCurDosing = function() {
    this.curDosing = "array" == typeof this.dosing || this.dosing instanceof Array ? this.dosing[0][1] : this.dosing;
};

MedBland.prototype.calcDose = function() {
    if ("string" == typeof this.dosing || this.dosing instanceof String) return this.dosing;
    if ("array" == typeof this.dosing || this.dosing instanceof Array) {
        var dose = this._calcDose();
        var strengths = '<select id="' + this.getID() + '-sel"';
        strengths += ' data-native-menu="false">';
        for (var i = 0; i < this.dosing.length; ++i) {
            strengths += '<option value="' + this.dosing[i][1] + '"';
            this.dosing[i][1] === this.curDosing && (strengths += ' selected="selected"');
            strengths += ">" + this.dosing[i][0] + "</option>";
        }
        strengths += "</select>";
        return strengths + " " + dose;
    }
    return this.dosing;
};

MedBland.prototype._calcDose = function() {
    try {
        var val = $("#" + this.getID() + "-sel").val();
        val.length > 0 && (this.curDosing = val);
    } catch (e) {}
    return this.curDosing;
};